
from math import sqrt
from math import fabs
from math import ceil
from math import floor
from math import e
import random
from common import *

class Leaf(Sprite):

	def __init__(self, image_data, **kwargs):
		self.y_velocity = random.randint(-20, 20)
		self.x_velocity = random.randint(-20, 20)
		self.affection_factor = random.random() * 0.2 + 0.1
		self.sticky_factor = random.random() * 1 + 1
		self.side_boost = 10
		self.in_blob = False
		self.blob = None
		Sprite.__init__(self, "", image_data, **kwargs)
		
		self.font = font.load('Arial', 10)
		self.info_text = font.Text(self.font, y=self.y, x=self.x)
		

	def update(self, wind_map):
		curr_wind = wind_map.get_velocity(self.x, self.y)
		
		# Apply wind to velocity
		self.y_velocity = (self.y_velocity + curr_wind[1] * self.affection_factor) / (0.99 + self.affection_factor)
		self.x_velocity = (self.x_velocity + curr_wind[0] * self.affection_factor) / (0.99 + self.affection_factor)
		
		# Apply stickyness to velocity
		self.y_velocity -= self.y_velocity * ( e**( (-fabs(self.y_velocity))/self.sticky_factor ) )
		self.x_velocity -= self.x_velocity * ( e**( (-fabs(self.x_velocity))/self.sticky_factor ) )
		
		# Apply velocity to position
		self.y += self.y_velocity
		self.x += self.x_velocity
		
		#Have we left the screen?
		if (self.y < 0):
			self.y += 600
			self.y_velocity -= self.side_boost
		if (self.y >= 600):
			self.y -= 600
			self.y_velocity += self.side_boost
		if (self.x < 0):
			self.x += 800
			self.x_velocity -= self.side_boost
		if (self.x >= 800):
			self.x -= 800
			self.x_velocity += self.side_boost
	
	def attach_blob(self, blob, notify_blob):	# Triggered when leaf enters a blob
		self.in_blob = True
		self.blob = blob
		if (notify_blob):
			self.blob.attach_leaf(self)
	
	def unattach_blob(self, notify_blob):		# Triggered when leaf leaves its blob
		self.in_blob = False
		if (notify_blob):
			self.blob.unattach_leaf(self)
		self.blob = None
	
	def set_velocity(self):		# DEPRECATED - UNUSED
		self.x_velocity = random.randint(-3,3)
		self.y_velocity = random.randint(-3,3)

