
from math import sqrt
from math import fabs
from math import ceil
from math import floor
import random
from common import *

class Windparticle(Sprite):

	def __init__(self, Windplayer, image_data, **kwargs):
		self.power = random.random() * 2 + 1.5#makes sure that the particles moves differently
		Sprite.__init__(self, "", image_data, **kwargs)
		self.x_velocity = 0
		self.y_velocity = 0
		self.x_offset = random.randint(-30, 30)
		self.y_offset = random.randint(-30, 30)
		self.clockwise = random.randint(0,1)
	
	#used by collision prevention
	def add_velocity(self, x, y):
		self.x_velocity += limit(x, 5)
		self.y_velocity += limit(y, 5)
	
	#Used with Windplayer
	def add_velocity(self, x, y):
		self.x_velocity = self.x_velocity * 0.9 + x
		self.y_velocity = self.y_velocity * 0.9 + y
	
	def draw(self):
		Sprite.draw(self)
	
	def update(self, Windplayer, index, list, Windmap):
		#update location
		self.x += self.x_velocity
		self.y += self.y_velocity
		
		#interact with each other
		#for i in range(index + 1, len(list)):
		#	len_x = self.x - list[i].x
		#	len_y = self.y - list[i].y
		#	#no need to prevent a collision if it isn't going to happen
		#	if fabs(len_x) < 30 and fabs(len_y) < 30:
		#		lenght = sqrt(len_x * len_x + len_y * len_y)
		#		self.add_velocity(len_x * self.power / lenght, len_y * self.power / lenght)
		#		list[i].add_velocity(-len_x * list[i].power / lenght, -len_y * list[i].power / lenght)
		
		#interact with player
		len_x = self.x - Windplayer.x + self.x_offset
		len_y = self.y - Windplayer.y + self.y_offset
		lenght = sqrt(len_x * len_x + len_y * len_y)
		#moves the Windparticle towards the player
		self.add_velocity((-len_x / lenght) * self.power, (-len_y / lenght) * self.power)
		#keeps the particles rotating
		if self.clockwise:
			self.add_velocity(len_y * self.power / (lenght * lenght), -len_x * self.power / (lenght * lenght))
		else:
			self.add_velocity(-len_y * self.power / (lenght * lenght), len_x * self.power / (lenght * lenght))
		
		#updates the wind map
		Windmap.add_velocity(self.x, self.y, self.x_velocity, self.y_velocity)

class Windplayer(Sprite):

	def __init__(self, text_x, text_y, **kwargs):

		self.points = 0
		Sprite.__init__(self, "player.png", **kwargs)
		
		self.x_dir = 0
		self.y_dir = 0
		self.x_old = self.x
		self.y_old = self.y

		#Create a font for our kill message
		self.font = font.load('Arial', 28)
		#The pyglet.font.Text object to display the kills
		self.kill_text = font.Text(self.font, y=text_y, x=text_x)
		self.text_x = text_x
	
	def update(self, Windmap):
		self.x_dir = (self.x - self.x_old)
		self.y_dir = (self.y - self.y_old)
		self.x_old = self.x
		self.y_old = self.y
		#if self.x_dir != 0 or self.y_dir != 0:
		#	Windmap.blow(self.x, self.y, self.x_dir, self.y_dir, 30)
		self.x_dir = 0
		self.y_dir = 0
	
	def draw(self):
		Sprite.draw(self)
		self.kill_text.x = self.text_x - (len("%d" % self.points)-1) * 22 
		self.kill_text.text = ("Points: %d") % (self.points)
		self.kill_text.draw()

	def add_points(self, addition):
		self.points += addition

class Windmap():
	
	def __init__(self, width, height, size):
		self.size = size
		self.map = []
		for i in range(int(ceil(width / self.size))):
			self.map.append([])
			for j in range(int(ceil(height / self.size))):
				self.map[i].append([])
				self.map[i][j].append(0)
				self.map[i][j].append(0)
		print len(self.map)
		print len(self.map[0])
		print len(self.map[0][0])
		#the result is neaded to avoid conflicts between old and new values in update
		self.result = self.map
	
	def add_velocity(self, x, y, x_velocity, y_velocity):
		x = int(x / self.size)
		y = int(y / self.size)
		if x >= 0 and x < len(self.map) and y >= 0 and y < len(self.map[0]):
			self.map[x][y][0] += x_velocity * 2
			self.map[x][y][1] += y_velocity * 2
		
	def get_velocity(self, x, y):
		"""Get the wind speed on the current spot. Is used to blow leaves and wind particles around"""
		x = int(x / self.size)
		y = int(y / self.size)
		
		# If the current spot is outside the windmap, use the velocity at the nearest point
		if x >= 0:
			if x >= len(self.map):
				x = len(self.map) - 1
		else:
			x = 0
		
		if y >= 0:
			if y >= len(self.map[0]):
				y = len(self.map[0]) - 1
		else:
			y = 0
		
		return [ self.map[x][y][0], self.map[x][y][1] ]
		
	
	def blow(self, x_pos, y_pos, x_dir, y_dir, radius):
		#make a single blow in one direction
		for x in range(int(round((x_pos - radius) / self.size)), int(round((x_pos + radius) / self.size))):
			for y in range(int(round((y_pos - radius) / self.size)), int(round((y_pos + radius) / self.size))):
				if x > 1 and x < len(self.map) - 1 and y > 1 and y < len(self.map[0]) - 1:
					self.map[x][y][0] += limit(x_dir, 50)
					self.map[x][y][1] += limit(y_dir, 50)
		
	
	def update(self):
		#calculates the new results
		for x in range(len(self.map)):
			for y in range(len(self.map[x])):
				self.result[x][y][0] = (self.map[(x + 1) % len(self.map)][y][0] + self.map[x - 1][y][0] + self.map[x][(y + 1) % len(self.map[x])][0] + self.map[x][y - 1][0]) / 4.05 #divide by 4.001 (instead of 4) to slowly kill the wind
				self.result[x][y][1] = (self.map[(x + 1) % len(self.map)][y][1] + self.map[x - 1][y][1] + self.map[x][(y + 1) % len(self.map[x])][1] + self.map[x][y - 1][1]) / 4.05
		#moves the results from last calculation to the calculation array
		self.map = self.result
	
	def draw(self):
		v2i = []
		for x in range(len(self.result)):
			for y in range(len(self.result[x])):
				v2i.append(x * self.size)
				v2i.append(y * self.size)
				v2i.append(x * self.size + int(self.result[x][y][0]) / 2)
				v2i.append(y * self.size + int(self.result[x][y][1]) / 2)
		graphics.draw(len(self.result) * len(self.result[0]) * 2, gl.GL_LINES, ('v2i', v2i))


